#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
	Dummy class for offline testing
	
	Has no functionality other than holding values and simulating some state changes
"""

import random
import time
from thread import start_new_thread


# Make sure we have all constants
from vr2 import *


STATUS_INTERVAL = 0.5

def put(txt):
	print('vr2dummy:\t' + str(txt))

class VR2Dummy:
	def __init__(self, serial_port=ARDUINO_SERIAL_PORT, serial_baud=ARDUINO_SERIAL_BAUD, autostart=True):
		# The fun stuff (parsed and usable values)
		self.is_connected = False
		self.is_ready = False
		
		self.battery = 0.0	# 0.0 ... 1.0
		self.gear = 0	# 0..VR2_MAX_GEAR
		self.speaker = 0	# 0..3 for different beeps
		self.joy = [0.0, 0.0]	# -1.0 ... +1.0
		
		self.on_status = None
		
		# GPS Compass
		self.pos_geo = [0.0, 0.0]
		self.on_pos_geo = None
		
		self.magnet = [0.0, 0.0, 0.0]
		self.on_magnet = None
		self.heading = 0.0
		self.on_heading = None
		
		
		# Faking stuff
		self.fake_power_on = False
		
		self.running = False
		if autostart:
			self.start()
	
	def start(self):
		start_new_thread(self.run_sim, ())
	
	def run_sim(self):
		self.running = True
		
		while self.running:
			
			time.sleep(STATUS_INTERVAL)
			
			# Fake status
			self.battery = 0.5 + random.random() * 0.1
			
			# Fake power up
			if (self.fake_power_on):
				if (not self.is_connected):
					self.is_connected = True
					# Continue on next update
				else:
					if (not self.is_ready):
						self.is_ready = True
						self.fake_power_on = False
			
			# Notify
			if not self.on_status is None:
				self.on_status()
	
	def kill(self):
		put('kill()')
		self.is_ready = False
		self.is_connected = False
		self.running = False
	
	def blink(self):
		put('blink()')
		pass
	
	def power_on(self, wait=True):
		put('power_on()')
		self.is_ready = False
		self.is_connected = False
		
		time.sleep(0.5)
		
		#self.is_connected = True
		#self.is_ready = True
		self.fake_power_on = True
	
	def power_off(self):
		put('power_off()')
		self.is_ready = False
		self.is_connected = False
	
	def set_joystick(self, x, y):
		put('set_joystick(%f, %f)' % (x, y))
		self.joy = [x,y]
	
	def press_key(self, key):
		put('press_key(%02X)' % (key))
		if (key == VR2_POWER):
			self.power_off()
		if (key == VR2_KEY_FASTER):
			self.gear += 1
		if (key == VR2_KEY_SLOWER):
			self.gear -= 1
		
	
	def stop(self):
		put('stop()')
		self.joy = [0.0, 0.0]
	
	def set_gear(self, gear):
		put('set_gear(%d)' % (gear))
		self.gear = max(0, min(VR2_MAX_GEAR, gear))
	
